 
function [results] = MVN_CrossVal(file,num_out,num_sect,p_train_data,nfold,numN,local_tr,ranran,Giunto,app)

% file = file of samples
% num_out = Num. of outputs
% p_train_data = perc. of training data
% nfold = N of fold for cross validation
% numN = neurons number;
% local_tr = soglia locale per soft margin
% ranran = random range for neurons weights
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Data = load(file);

N = size(Data,1);
fold_dim = floor(N/nfold);
col_Idx = randperm(N);

% selezione indici per cross validation
test_Idx = zeros(nfold,fold_dim);
train_Idx = zeros(nfold,fold_dim * (nfold-1));
for i=1:nfold
    test_Idx(i,:) = col_Idx((i-1)*fold_dim+1 : i*fold_dim);
end
for i=1:nfold
    for j=1:i-1
       train_Idx(i,(j-1)*fold_dim+1:j*fold_dim) = test_Idx(j,:);
    end
    for j=i+1:nfold
       train_Idx(i,(j-2)*fold_dim+1:(j-1)*fold_dim) = test_Idx(j,:);    
    end
end
% -----------------------------------------

results = zeros(nfold,num_out,3);  
max_ver = 0.0;
max_learn = 0.0;
    
for NFold = 1:nfold

    app.Label_3.Text = "NFold: " + num2str(NFold);
    % nur = number of samples
    [nur,nuc] = size(Data);
     
    % ning = number of complex inputs
    ning = (nuc - num_out)/2;
    
    datamax=max(Data);
    DataC = zeros(nur,ning);
    
    for k = 1:ning
        %normalizzazione moduli
        DataC(:,k) = (Data(:,2*k-1)/abs(datamax(2*k-1))).*exp(1i * Data(:,2*k));
        
        %normalizzazione moduli e fasi
        %DataC(:,k) = (Data(:,2*k-1)/abs(datamax(2*k-1))).*exp(1i * (Data(:,2*k)/abs(datamax(2*k))));
    end
    
    %copying desired output values
    
    %copying desired output values
    for i=1:nur
        if Giunto==0
            for k=1:num_out
                val = Data(i,nuc-num_out+k);
                if val==2
                    DataC(i,ning+k*2-1) = 1;
                elseif val==3
                    DataC(i,ning+k*2-1) = 1;
                    DataC(i,ning+k*2) = 1;
                end
            end
        else
            val = Data(i,nuc-num_out+Giunto);
            if val==2
                DataC(i,ning+1) = 1;
            elseif val==3
                DataC(i,ning+1) = 1;
                DataC(i,ning+2) = 1;
            end
        end
    end
    
    %Creation of the learning set
    LearningData = DataC(train_Idx(NFold,:),:);    
    %Creation of the testing set
    TestingData = DataC(test_Idx(NFold,:),:);

    % Learning
    
    if Giunto==0
        nout=num_out*2;
    else
        nout=2;
    end
    
    for i = 1:nout
        sec_nums(i) = num_sect;
    end
    tic;
    
    [max_class_rate, OutputL, OutputV] = Net_learn(LearningData, TestingData, numN, nout, sec_nums, 0.05, local_tr, 30, ranran, app);
    
    results(NFold,1,3) = toc;
    max_ver = max_ver + max_class_rate.ver;
    max_learn = max_learn + max_class_rate.learn;
    for ij = 1:nout
        results(NFold,ij,1) = OutputV.c_rates(ij);
        results(NFold,ij,2) = OutputL.c_rates(ij);
    end
    if (app.Stop == 1)
        break;
    end
end

%output results
clRateVer = zeros(nout,1);
clRateLearn = zeros(nout,1);
tempo = 0.0;

lrmax = zeros(nout,1);
lrmin = zeros(nout,1);
lrmin = lrmin + 1.0;

for nf = 1:NFold
    tempo = tempo + results(NFold,1,3);
    for ij = 1:nout       
        clRateVer(ij,1) = clRateVer(ij,1) + results(nf,ij,1);
        clRateLearn(ij,1) = clRateLearn(ij,1) + results(nf,ij,2);
        if results(nf,ij,1) > lrmax(ij,1)
            lrmax(ij,1) = results(nf,ij,1);
        end
        if results(nf,ij,1) < lrmin(ij,1)
            lrmin(ij,1) = results(nf,ij,1);
        end        
    end
end
tempo = tempo/NFold;
max_ver = max_ver/NFold;
max_learn = max_learn/NFold;
for ij = 1:nout
    clRateVer(ij,1) = clRateVer(ij,1)/NFold;
    clRateLearn(ij,1) = clRateLearn(ij,1)/NFold;
end

L_mode = 0;
smode = 'BackProp';
if app.NoProp.Value==1
    L_mode=1;
    smode = 'NoProp';
elseif app.BPnew.Value == 1;
    L_mode=2;
    smode = 'BPnew';
end
NumG = str2double(app.NGiuntiDropDown.Value);
Giunto = app.GiuntoEditField.Value;
msg = sprintf("N. Joints: %d - Joint N. %d - %d neurons - %s\nCross-Validation  (",NumG,Giunto,numN,smode);
msg = msg + sprintf("mwgt=%7.2f locTr=%7.2f LRate=%7.2f bistop=%d)\n",app.ModWgtEditField.Value,app.LocTrEditField.Value,app.LRateEditField.Value,app.nobestiterations.Value);

msg = msg + sprintf("\ntraining: (cl. rate: %7.4f)\n", max_learn);
for ij=1:nout
        msg = msg + sprintf ("out %02d -> cl_rate: %7.4f \n", ij, clRateLearn(ij,1));
end
msg = msg + sprintf("\ntesting: (cl. rate: %7.4f)\n", max_ver);
for ij=1:nout
        msg = msg + sprintf ("out %02d -> cl_rate: %7.4f (min: %7.5f max: %7.5f)\n", ij, clRateVer(ij,1),lrmin(ij,1),lrmax(ij,1));
end
msg = msg + sprintf('\ntime: %7.5f sec\n',tempo);
app.TextArea.Value = msg;