function [max_class_rate, BestOutL, BestOutV] = Net_learn(Input, TestD, hidneur_num, outneur_num, sec_nums, RMSE_thresh, local_thresh, errc_thresh, ranran, app)

%Input = matrix containing a learning set. Each row is an input sample
%TestD = matrix containing a testing set. Each row is an input sample
%followed by desired output(s). There should be outneur_num desired outputs
%there (the same amount as the number of output neurons)

%hidneur_num = number of hidden neurons
%outneur_num = number of output neurons

%sec_nums = (1 x outneur_num) vector containing the number of sectors in
%each output neuron

%local_thresh = local angular threshold for determining error

%RMSE_thresh - tolerance threshold for angular RMSE 
%local_thresh - angular tolerance threshold to determine when a current
%output should be considered "incorrect" in terms of deviation from the
%bisector of the desired output sector

tic;
%Use the clock to set the stream of random numbers
%RandStream.setDefaultStream(RandStream('mt19937ar','seed',sum(100*clock)));
RandStream.setGlobalStream(RandStream('mt19937ar','seed',sum(100*clock)));

% Determines the number of columns in the Input
n = size(Input, 2);
% Extraction of inputs from Input
X = Input(:,1:n-outneur_num);
% Extraction of desired outputs from Input (outneur_num columns)
y_d = Input(:,n-outneur_num+1 : n);


%Determine the number of learning samples
N = size(X, 1);

%threshold of non-effective iterations
iter_sog = app.nobestiterations.Value;
last_iter_ok = 0;

%Determine the number of input variables n
n = size(X, 2);

coef_w = ranran; % range 0.5*coef_w
%Generate random weights for the hidden neurons:
hidneur_weights = zeros(n+1, hidneur_num);

for hh = 1 : hidneur_num

%     %real part (interval -0.5 to 0.5)
%     w_re = (rand(n+1, 1) - 0.5)*coef_w;
%     %imaginary part (interval -0.5 to 0.5)
%     w_im = (rand(n+1, 1) - 0.5)*coef_w;
% 
%     %Construct a weights vector, dimensions (n+1 x 1)
%     hidneur_weights(:, hh) = w_re + 1i .* w_im;
 
%       forma polare
      w_m = rand(n+1, 1) * coef_w;
      w_p = rand(n+1, 1) * 2.0 * pi;
      hidneur_weights(:, hh) = w_m .* exp(1i * w_p);
        

end


%Generate random weights for the output neurons:
outneur_weights = zeros(hidneur_num+1, outneur_num);

for pp = 1 : outneur_num
    
%     %real part (interval -0.5 to 0.5)
%     w_re = (rand(hidneur_num+1, 1) - 0.5)*coef_w;
%     %imaginary part (interval -0.5 to 0.5)
%     w_im = (rand(hidneur_num+1, 1) - 0.5)*coef_w;
%     
%     %Construct a weights vector, dimensions (hidneur_num+1 x 1)
%     outneur_weights(:, pp) = w_re + 1i .* w_im;

%       forma polare
      w_m = rand(hidneur_num + 1, 1) * coef_w;
      w_p = rand(hidneur_num + 1, 1) * 2.0 * pi;
      outneur_weights(:, pp) = w_m .* exp(1i * w_p);

end


%----

%Convert desired network output values (y_d), given as class labels, into
%desired phase values (phase_d):
phase_d = zeros(N, outneur_num);
for pp = 1 : outneur_num   
    phase_d(:, pp) = y_d(:, pp) .* (2*pi/sec_nums(pp));
end

%Ensure that the angular range is [0, 2pi) instead of (-pi, pi)
for ii=1:N
    for pp = 1 : outneur_num
        
        if (phase_d(ii, pp) < 0)
            phase_d(ii, pp) = phase_d(ii, pp) + 2*pi;
        end
    end
end

%Determine sector size (angle), separately for each output neuron.
%sec_size is a (1 x outneur_num) vector
sec_size = 2*pi ./ sec_nums;

%Shift all desired phase values by half-sector counter-clockwise
for pp = 1 : outneur_num   
    phase_d(:, pp) = phase_d(:, pp) + sec_size(pp)/2;
end


%Construct a matrix of desired network outputs (lying on the unit circle)
znet_d = exp(1j .* phase_d);


%append a column of 1s to X from the left, yielding a (N x n+1) matrix
%app_X
col_app(1:N) = 1;
col_app = col_app.';
app_X = [col_app X];

%Construct the pseudo-inverse of X

X_pinv = pinv(app_X);

iterations = 1;
nesovpad = 1;

min_nesovpad = N;
%max_class_rate.learn = 0;
max_class_rate.ver = -1;

LearnFlag = 1;


min_err_all = 10^5; % bitmax;
min_RMSE = 10^5; % bitmax;

N_x_outneur_num = N * outneur_num;

InputTestD = Input;

L_mode = 0;
smode = 'BackProp';
if app.NoProp.Value==1
    L_mode=1;
    smode = 'NoProp';
elseif app.BPnew.Value == 1;
    L_mode=2;
    smode = 'BPnew';
end

NumG = str2num(app.NGiuntiDropDown.Value);
Giunto = app.GiuntoEditField.Value;
%----------------------------------------------------------------------------------------------------------------------
%----------------------------------------------------------------------------------------------------------------------
% tit = sprintf('N. Joints: %d - Joint N. %d - %s',NumG,Giunto,smode);
%----------------------------------------------------------------------------------------------------------------------
%----------------------------------------------------------------------------------------------------------------------
tit = sprintf('N. Sections: %d - Section N. %d - %s',NumG,Giunto,smode);
app.Label_2.Text=tit;
lamb = app.LRateEditField.Value;
lrdec = app.LrDecEditField.Value;
nbp=0;

while ( LearnFlag == 1) 
    
    hh = app.OutputNCBox.Value;
    if (hh == 1)
            compass(outneur_weights);            
    end
    hh =app.HiddenNCBox.Value;
    if (hh == 1)
            compass(hidneur_weights(:,1:20));            
    end
    
              
    if L_mode==0            % normal Back Prop       
        %Compute the output of hidden neurons for all samples
        hid_outmat = app_X * hidneur_weights;
        
        % Calculation the absolute values of the
        % current hidden neurons weighted sums
        abs_hid_outmat = abs(hid_outmat);
        
        %Move outputs to the unit circle
        hid_outmat = hid_outmat ./ abs_hid_outmat;
        
        %Determine the network error
        [hid_errmat] = ErrBackProp(hid_outmat, outneur_weights, phase_d, znet_d, y_d, sec_size, N, hidneur_num, outneur_num, local_thresh, abs_hid_outmat);
        
        %Adjust weights of hidden neurons
        hidneur_weights = HidNeuron_weightadj(X, X_pinv, hidneur_weights, hid_errmat, N);

    elseif L_mode == 2
        
        %Adjust weights of hidden neurons
        lrrr = lamb * (lrdec^iterations);
        hidneur_weights = HidNeurBack_weightadj(app_X, hidneur_weights, lrrr, outneur_weights, phase_d, znet_d, y_d, sec_size, outneur_num, local_thresh);      
        
    end
    
    %Compute the output of hidden neurons for all samples
    hid_outmat = app_X * hidneur_weights;
    
    %Move outputs to the unit circle
    hid_outmat = hid_outmat ./ abs(hid_outmat);
    
    %Adjust weights of output neurons
    [outneur_weights, z_outneur] = OutNeuron_weightadj(hid_outmat, outneur_weights, phase_d, znet_d, y_d, sec_size, N, outneur_num, local_thresh,app);
       
    %Compute and display learning statistics----
    iterations = iterations + 1;     
    
    %Determine the number of nesovpad and angular RMSE
    current_phase = angle(z_outneur);
    
    for ii=1:N

        for pp = 1 : outneur_num

            if (current_phase(ii, pp) < 0)
                current_phase(ii, pp) = current_phase(ii, pp) + 2*pi;
            end
        end
    end

    current_labels = zeros(N, outneur_num);
    for pp = 1 : outneur_num        
        current_labels(:, pp) = floor(current_phase(:, pp) ./ sec_size(pp));
    end

    sovpad = 0;
    ang_RMSE = 0;
    
    for ii=1:N
        if (current_labels(ii,:) == y_d(ii,:))
            sovpad = sovpad + 1 ;
        end
    end
    nesovpad = N - sovpad;
    
    for ii=1:N
    
        for pp = 1 : outneur_num
            
            ang_err = abs(current_phase(ii, pp) - phase_d(ii, pp));

            if (ang_err > pi)
                ang_err = 2*pi - ang_err;
            end
            
            ang_RMSE = ang_RMSE + ang_err^2;

        end
        
    end
   
    ang_RMSE = sqrt(ang_RMSE / N_x_outneur_num);
    
    if (ang_RMSE < min_RMSE)
        min_RMSE = ang_RMSE;
    end
    
    %If nesovpad == 0, stop learning
    if ( (nesovpad <= errc_thresh)  && (ang_RMSE < RMSE_thresh) )
        
        LearnFlag = 0;
    end
    
    if (nesovpad < min_nesovpad)
        %max_class_rate.learn = classif_rate.learn;
        min_nesovpad = nesovpad;
    end
    
    [classif_rate.ver,OutV,msgV,y_dn,current_labelsn] = Net_test(TestD, hidneur_weights, outneur_weights, sec_nums);
    [classif_rate.learn,OutL,msgL] = Net_test(InputTestD, hidneur_weights, outneur_weights, sec_nums);
    
    [nrmc,ncmc]=size(y_dn);
    MC=zeros(2*ncmc,nrmc);
    for i=1:ncmc
        MC(2*i-1,:)=y_dn(:,i)';
        MC(2*i,:)=current_labelsn(:,i)';
    end
        
    if(classif_rate.ver > max_class_rate.ver) 
                     last_iter_ok = iterations;
                     max_class_rate.ver = classif_rate.ver;
                     max_class_rate.learn = classif_rate.learn;                   
                     
                     msg = tit;
%------------------------------------------------------------------------------------------------------------------------
%------------------------------------------------------------------------------------------------------------------------
%                      msg = msg + sprintf("\n\n%d Neurons - best iteration = %d\n\n", hidneur_num, iterations);
%------------------------------------------------------------------------------------------------------------------------
%------------------------------------------------------------------------------------------------------------------------
                     msg = msg + sprintf("\n\n%d Nsections - best iteration = %d\n\n", hidneur_num, iterations);
                     msg = msg + "training:    ";
                     msg = msg + msgL + sprintf('\n');
                     msg = msg + "testing:     ";
                     msg = msg + msgV;
                     app.TextArea.Value = msg;
                     
                     BestOutL = OutL;
                     BestOutV = OutV;
                     
                     r=0;
                     for i=1:2:2*ncmc
                     matriceconff(i:i+1,:)=confusionmat(MC(i,:),MC(i+1,:),'order',[1 0]);
                     r=r+1;
                     end
                     
                     classi=['0';'1'];
%----------------------------------------------------------------------------------------------------------------------
%----------------------------------------------------------------------------------------------------------------------
%                      if app.ConfusionMatrix.Value==1
%                      r=0;
%                      t=0;
%                      w=1;
%                     for i=1:2:2*ncmc
%                      matriceconf=confusionmat(MC(i,:),MC(i+1,:),'order',[1 0]);
%                      r=r+1;
%                      t=t+1;
%                      if t==3
%                         t=1;
%                         w=w+1;
%                      end 
%                      figure(1);subplot(3,2,r)
%                      confusionchart(matriceconf,classi);
%                      title(sprintf("Neuron N. %d  Joint N. %d\n", t, w))
%                     end
%                      end
%----------------------------------------------------------------------------------------------------------------------
%----------------------------------------------------------------------------------------------------------------------
    end

    x_x(iterations) = iterations;
    rate_plot(iterations) = classif_rate.ver;
    lrn_plot(iterations) = classif_rate.learn;
    x_best(1) = last_iter_ok;
    y_best(1) = max_class_rate.ver;
    plot(app.UIAxes1,x_x,rate_plot,x_x,lrn_plot,x_best,y_best,'mo');
    legend(app.UIAxes1,'verif.','train.','location','southwest');

    bar_x=1:outneur_num;
    
    pc_rate=[BestOutL.c_rates',BestOutV.c_rates',OutV.c_rates'];
    
    bar(app.UIAxes2,pc_rate);
    legend(app.UIAxes2,'best train.','best ver.','Current ver.','location','southwest');

    %check over non-effective iterations
    if ((iterations - last_iter_ok) > iter_sog)
        LearnFlag = 0;
    end
    
    %Display the statistic in a separate figure
    app.IterLab.Text=sprintf('%d  (best: %d)',iterations,last_iter_ok);
    app.ClRateLab.Text=num2str(classif_rate.ver);
    app.BestClRateLab.Text=num2str(max_class_rate.ver);
    app.RMSELab.Text=num2str(ang_RMSE);
    drawnow;
    
    if (app.Stop == 1)
        LearnFlag = 0;       
    end
end

classi=['0';'1'];
%-----------------------------------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------------------------------
% r=0;
% t=0;
% w=1;
% for i=1:2:2*ncmc
%  r=r+1;
%  t=t+1;
%  if t==3
%     t=1;
%     w=w+1;
%  end 
%  figure(1);subplot(3,2,r)
%  confusionchart(matriceconff(i:i+1,:),classi);
%  title(sprintf("Neuron N. %d  Joint N. %d\n", t, w))
% end
%-----------------------------------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------------------------------

msg = msg + sprintf("\nTime: %10.5f\n",toc);
app.TextArea.Value = msg;



