
function [classif_rate, Output, msg,current_labels,y_d] = Net_test(Input, hidneur_weights, outneur_weights, sec_nums)

%Input = matrix containing a test set. Each row is an test sample
%followed by a desired output. Unlike in the learning case even for
%multiple output neurons there should be a single (multi-valued) desired
%output there

%hidneur_weights = weights of hidden layer neurons 
%outneur_weights = weights of output layer neurons 
%num_sect = num. of sector for neuron
%for multiple output neurons performing binary classification

%Determine the number of output neurons
outneur_num = size(outneur_weights, 2);
% Determines the number of columns in the Input
n = size(Input, 2);
% Extraction of inputs from Input
X = Input(:,1:n-outneur_num);
% Extraction of desired outputs from Input
y_d = Input(:,n-outneur_num+1:n);

%Determine the number of learning samples
N = size(X, 1);

%append a column of 1s to X from the left
%app_X
col_app(1:N) = 1;
col_app = col_app.';
app_X = [col_app X];

%Compute the output of hidden neurons for all samples
hid_outmat = app_X * hidneur_weights;

%Move outputs to the unit circle
hid_outmat = hid_outmat ./ abs(hid_outmat);

%append a column of 1s to hid_outmat
hid_outmat = [col_app hid_outmat];

%Compute the output of the network
z_outneur = hid_outmat * outneur_weights;

sec_size = 2*pi ./ sec_nums;

current_phase = angle(z_outneur);
current_phase = mod(current_phase, 2*pi);

current_labels = zeros(N, outneur_num);

 for pp = 1 : outneur_num        
          current_labels(:, pp) = floor(current_phase(:, pp) ./ sec_size(pp));
 end

numok = 0;


c_rates = zeros(1,outneur_num);
sovpad = zeros(outneur_num,1);
fneg = zeros(outneur_num,1);
fpos = zeros(outneur_num,1);

for ii=1:N
    if (current_labels(ii,:) == y_d(ii,:))
        numok = numok + 1;
    end
    for ij=1:outneur_num
        if (current_labels(ii,ij) == y_d(ii,ij))
            sovpad(ij) = sovpad(ij) + 1;
        else
            if(y_d(ii,ij) == 0.0)
                fpos(ij) = fpos(ij) +1;
            else
                fneg(ij) = fneg(ij) +1;
            end
        end
        c_rates(ij)=sovpad(ij)/N;
    end
    
end
classif_rate = numok / N;
toterr = N-numok;

msg = sprintf ("Classification Rate = %7.4f (tot: %d )  Errors: %d\n", classif_rate, N, toterr);
for ij=1:outneur_num
    msg = msg + sprintf ("out %02d -> ok: %04d  err: %04d (fpos: %04d fneg: %04d) cl_rate: %7.4f \n", ij, sovpad(ij), N - sovpad(ij), fpos(ij), fneg(ij), sovpad(ij)/N);
end
Output.ClassifRate = classif_rate;
Output.Actual = current_labels;
Output.Desired = y_d;
Output.c_rates = c_rates;
% fprintf("---------------------------");

